local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local OPTIONS = Lib.options;
local DEBUG = Lib.debug;
local module = Lib.multiplanner

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");

--Map of arrays to each missions party layout
--Used to determine the position that each follower is displayed within a party
-- parties[misisonID][1..3]
local parties = {}
module.parties = parties

local COLLAPSED_MISSION_BUTTON_HEIGHT = 85


local function getFollowerMissionBiasRGB(missionID,followerID)
	local bias = C_Garrison.GetFollowerBiasForMission(missionID, followerID);

	if ( bias == -1 ) then -- Impossible
		return 1, 0.1, 0.1;
	elseif ( bias < 0 ) then -- Hard
		return 1, 0.5, 0.25;
	else -- Normal
		return 1, 1, 1;
	end
end

local function GarrisonMissionPlanner_CheckCounter(button, counterID)
	for i = 1, #button.Mechanics do
		if ( counterID == button.Mechanics[i].mechanicID and not button.Mechanics[i].hasCounter ) then
			button.Mechanics[i].hasCounter = true;
			button.Mechanics[i].Check:Show();
			return;
		end
	end
end

function GarrisonMissionPlannerMinimizedFollower_OnMouseUp(self, button)
	if ( button == "RightButton" ) then
		if ( self.info ) then
			GarrisonMissionPlanner_ClearFollower(self, true);
		end
	else
		GarrisonMissionPlannerButton_OnClick(self:GetParent(), button)
	end
end

function GarrisonMissionPlannerButton_OnClick(self, button)
	local mission = self.info;
	if ( IsModifiedClick("CHATLINK") ) then
		local missionLink = C_Garrison.GetMissionLink(mission.missionID);
		if (missionLink) then
			ChatEdit_InsertLink(missionLink);
		end
		return;
	end

	PlaySound("UI_Garrison_CommandTable_SelectMission");
	if ( module.IsMissionSelected(mission) ) then
		GarrisonMissionPlanner_DeselectMission() -- Deselect misison
	else
		GarrisonMissionPlanner_SelectMission(mission) -- Select mission
	end
end

function GarrisonMissionPlannerMechanic_OnEnter(self)
	if ( self.info ) then
		GarrisonMissionMechanic_OnEnter(self)
	elseif ( self.environment ) then
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
		GameTooltip:SetText(self.environment);
		GameTooltip:AddLine(self.environmentDesc, 1, 1, 1, 1);
		GameTooltip:Show();
	end
end

function GarrisonMissionPlannerMechanic_OnLeave(self)
	GarrisonMissionMechanicTooltip:Hide();
	GameTooltip:Hide()
end

function GarrisonMissionPlannerFollowerFrame_OnMouseUp(self, button)
	if ( button == "RightButton" ) then
		if ( not self.info and button == "RightButton") then
			GarrisonMissionPlannerButton_OnClick(self:GetParent(), button)
		else
			GarrisonMissionPlanner_ClearFollower(self, true);
		end
	end
end

function module.clearFollowers(mission)
	for i = 1, mission.numFollowers do
		local party = parties[mission.missionID]
		if ( party and party[i] ) then
			C_Garrison.RemoveFollowerFromMission(mission.missionID, party[i])
			Lib.callbacks:Fire("FOLLOWER_REMOVED_FROM_MISSION", mission.missionID, i, "MultiPlanner")
		end
	end

	parties[mission.missionID] = {}
end

function GarrisonMissionPlanner_SetRewards(self, rewards, numRewards)
	if (numRewards > 0) then
		local index = 1;
		for id, reward in pairs(rewards) do
			if (not self.Rewards[index]) then
				self.Rewards[index] = CreateFrame("Frame", nil, self, "MultiPlannerListButtonRewardTemplate");
				self.Rewards[index]:SetPoint("RIGHT", self.Rewards[index-1], "LEFT", -4, 0);
			end
			local Reward = self.Rewards[index];
			Reward.Quantity:Hide();
			Reward.IconBorder:Hide();
			Reward.itemID = nil;
			Reward.currencyID = nil;
			Reward.tooltip = nil;
			if (reward.itemID) then
				Reward.itemID = reward.itemID;
				GarrisonMissionFrame_SetItemRewardDetails(Reward);

				local _, _, quality = GetItemInfo(reward.itemID);
				if  ( quality and quality > LE_ITEM_QUALITY_COMMON and BAG_ITEM_QUALITY_COLORS[quality]) then
					Reward.IconBorder:Show();
					Reward.IconBorder:SetVertexColor(BAG_ITEM_QUALITY_COLORS[quality].r, BAG_ITEM_QUALITY_COLORS[quality].g, BAG_ITEM_QUALITY_COLORS[quality].b);
				end

				if ( reward.quantity > 1 ) then
					Reward.Quantity:SetText(reward.quantity);
					Reward.Quantity:Show();
				end
			else
				Reward.Icon:SetTexture(reward.icon);
				Reward.title = reward.title
				if (reward.currencyID and reward.quantity) then
					Reward.currencyID = reward.currencyID;
					Reward.currencyQuantity = reward.quantity
					if (reward.currencyID == 0) then
						Reward.tooltip = GetMoneyString(reward.quantity);
						Reward.Quantity:SetText(BreakUpLargeNumbers(math.floor(reward.quantity / COPPER_PER_GOLD)));
						Reward.Quantity:Show();
					else
						Reward.Quantity:SetText(reward.quantity);
						Reward.Quantity:Show();
					end
				else
					Reward.tooltip = reward.tooltip;
				end

				if ( reward.followerXP and reward.followerXP > 0 ) then
					Reward.Quantity:SetText(BreakUpLargeNumbers(reward.followerXP));
					Reward.Quantity:Show();
				end
			end
			Reward:SetScale(0.8)
			Reward:Show();
			index = index + 1;
		end
	end

	for i = (numRewards + 1), #self.Rewards do
		self.Rewards[i]:Hide();
	end
end

function GarrisonMissionPlanner_SetFollowerPortrait(portraitFrame, followerInfo, showItemLevel)
	local color = ITEM_QUALITY_COLORS[followerInfo.quality];
	portraitFrame.PortraitRingQuality:SetVertexColor(color.r, color.g, color.b);
	portraitFrame.LevelBorder:SetVertexColor(color.r, color.g, color.b);
	if ( showItemLevel and followerInfo.level == GarrisonMissionFrame.followerMaxLevel ) then
		portraitFrame.Level:SetFormattedText(GARRISON_FOLLOWER_ITEM_LEVEL, followerInfo.iLevel);
	else
		portraitFrame.Level:SetText(followerInfo.level);
	end
	if ( followerInfo.displayID ) then
		GarrisonFollowerPortrait_Set(portraitFrame.Portrait, followerInfo.portraitIconID);
	end
end


local function updateMissionForParty(button)
	local mission = button.info
	if ( not mission ) then
		return
	end

	local location, xp, environment, environmentDesc, environmentTexture, locPrefix, isExhausting, enemies = C_Garrison.GetMissionInfo(mission.missionID);
	local totalTimeString, totalTimeSeconds, isMissionTimeImproved, successChance, partyBuffs, isEnvMechanicCountered, xpBonus, materialMultiplier, goldMultiplier = C_Garrison.GetPartyMissionInfo(mission.missionID)

	-- TIME
	if ( isMissionTimeImproved ) then
		totalTimeString = GREEN_FONT_COLOR_CODE..totalTimeString..FONT_COLOR_CODE_CLOSE
	elseif ( totalTimeSeconds >= GARRISON_LONG_MISSION_TIME ) then
		totalTimeString = format(GARRISON_LONG_MISSION_TIME_FORMAT, totalTimeString)
	end
	button.Summary:SetFormattedText(PARENS_TEMPLATE, totalTimeString)
	button.MissionTime:SetFormattedText(GARRISON_MISSION_TIME_TOTAL, totalTimeString)

	-- SUCCESS CHANCE
	button.Chance:SetFormattedText(PERCENTAGE_STRING, successChance)

	-- ENVIRONMENT
	if ( isEnvMechanicCountered ) then
		button.MissionEnvIcon.Check:Show()
	else
		button.MissionEnvIcon.Check:Hide()
	end

	-- MECHANICS
	for i = 1, #button.Mechanics do
		button.Mechanics[i].hasCounter = nil
		button.Mechanics[i].Check:Hide()
	end

	for i = 1, #button.Followers do
		local frame = button.Followers[i]
		local follower = frame.info
		if ( follower ) then
			if (C_Garrison.GetFollowerBiasForMission(mission.missionID, follower.followerID) > -1 ) then
				if (not follower.abilities) then
					follower.abilities = C_Garrison.GetFollowerAbilities(follower.followerID)
				end
				for a = 1, #follower.abilities do
					local ability = follower.abilities[a];
					for counterID, counterInfo in pairs(ability.counters) do
						GarrisonMissionPlanner_CheckCounter(button, counterID)
					end
				end
			end
		end
	end

	-- Update PartyBuffs
	local buffsFrame = button.PartyBuffs
	local buffCount = #partyBuffs;
	if ( buffCount == 0 ) then
		buffsFrame:Hide();
	else
		for i = 1, buffCount do
			local buff = buffsFrame.Buffs[i];
			if ( not buff ) then
				buff = CreateFrame("Frame", nil, buffsFrame, "GarrisonMissionPartyBuffTemplate");
				buff:SetPoint("LEFT", buffsFrame.Buffs[i - 1], "RIGHT", 8, 0);
			end
			buff.Icon:SetTexture(C_Garrison.GetFollowerAbilityIcon(partyBuffs[i]));
			buff.id = partyBuffs[i];
			buff:Show();
		end
		for i = buffCount + 1, #buffsFrame.Buffs do
			buffsFrame.Buffs[i]:Hide();
		end
		local width = buffCount * 28 + buffsFrame.BuffsTitle:GetWidth() + 40;
		buffsFrame:SetWidth(max(width, 160));
		buffsFrame:Show();
	end
	for i = buffCount + 1, #buffsFrame.Buffs do
		buffsFrame.Buffs[i]:Hide()
	end

	-- Rewards
	GarrisonMissionPage_UpdateRewardQuantities(button, materialMultiplier, goldMultiplier);

	-- XP
	if ( xpBonus > 0 ) then
		button.MissionXP:SetFormattedText(GARRISON_MISSION_BASE_XP_PLUS, xp + xpBonus, xpBonus);
		button.MissionXP.hasBonusBaseXP = true;
	else
		button.MissionXP:SetFormattedText(GARRISON_MISSION_BASE_XP, xp);
		button.MissionXP.hasBonusBaseXP = false;
	end
end

local counterCache = {};
local function getFollowerCounters(missionID, followerID)
	if ( not missionID or not followerID ) then
		return nil
	end

	if ( not counterCache[missionID]) then
		counterCache[missionID] = C_Garrison.GetBuffedFollowersForMission(missionID);

		--for id, traits in pairs(C_Garrison.GetFollowersTraitsForMission(missionID)) do
		--	if ( not counterCache[missionID][id] ) then
		--		counterCache[missionID][id] = traits
		--	else
		--		for i=1, #traits do
		--			tinsert(counterCache[missionID][id], traits[i])
		--		end
		--	end
		--end
	end

	return counterCache[missionID][followerID];
end

local counterLayout = {
	[1] = {
		[1] = { "LEFT", 64, -1 }
	},

	[2] = {
		[1] = { "LEFT", 64, -1 },
		[2] = { "LEFT", 1, "RIGHT", 16, 0 }
	},

	[3] = {
		[1] = { "LEFT", 64, -1 },
		[2] = { "LEFT", 1, "RIGHT", 16, 0 },
		[2] = { "LEFT", 2, "RIGHT", 16, 0 }
	}
}

local function applyCounterLayout(counter, layout, i)
	layout = layout[i]
	counter[i]:ClearAllPoints()

	if ( #layout == 3) then
		counter[i]:SetPoint(layout[1], layout[2], layout[3]);
	elseif ( #layout == 4) then
		counter[i]:SetPoint(layout[1], frame:GetParent(), layout[2], layout[3], layout[4]);
	elseif ( #layout == 5) then
		counter[i]:SetPoint(layout[1], counter[layout[2]], layout[3], layout[4], layout[5]);
	end
end

local function updateFollowerCounters(frame, counters)
	if (counters) then
		for i = 1, #frame.Counters do
			frame.Counters[i]:ClearAllPoints()
		end

		local layout = counterLayout[#counters]
		for i = 1, #counters do
			if (not frame.Counters[i]) then
				frame.Counters[i] = CreateFrame("Frame", nil, frame, "GarrisonMissionAbilityLargeCounterTemplate");
			end
			local Counter = frame.Counters[i];
			Counter.info = counters[i];
			Counter.info.showCounters = true;
			Counter.Icon:SetTexture(counters[i].icon);
			Counter.tooltip = counters[i].name;
			applyCounterLayout(frame.Counters, layout, i)
			Counter:Show();

			if( counters[i].traitID ) then
				Counter.Border:Hide()
			else
				Counter.Border:Show()
			end
		end
		for i = (#counters + 1), #frame.Counters do
			frame.Counters[i]:Hide();
		end
	else
		for i=1, #frame.Counters do
			frame.Counters[i]:Hide();
		end
	end
end

local function updateFollowerIcon(mission, frame, followerID)
	if ( followerID ) then
		local info = C_Garrison.GetFollowerInfo(followerID)
		frame.info = info

		local showItemLevel;
		if ( mission.iLevel > 600 and info.level == GarrisonMissionFrame.followerMaxLevel ) then
			frame.LevelBorder:SetAtlas("GarrMission_PortraitRing_iLvlBorder");
			frame.LevelBorder:SetWidth(70);
			showItemLevel = true;
		else
			frame.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
			frame.LevelBorder:SetWidth(58);
			showItemLevel = false;
		end

		frame:SetAlpha(1)
		frame.LevelBorder:Show()
		frame.Level:Show()
		GarrisonMissionPlanner_SetFollowerPortrait(frame, info, showItemLevel);

		local color = ITEM_QUALITY_COLORS[info.quality];
		frame.PortraitRingQuality:SetVertexColor(color.r, color.g, color.b);
	else
		frame.info = nil;

		frame:SetAlpha(0.7)
		frame.PortraitRingQuality:SetVertexColor(0.3, 0.3, 0.3);
		GarrisonFollowerPortrait_Set(frame.Portrait, nil);
		frame.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
		frame.LevelBorder:SetWidth(58);
		frame.Level:SetText("")
		frame.LevelBorder:Hide()
		frame.Level:Hide()
	end
end

local function updateFollowerFrame(mission, frame, followerID)
	if ( followerID == nil ) then
		frame.info = nil;
		frame.Class:Hide()
		frame.PortraitFrame.Empty:Show();
		frame.PortraitFrame.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
		frame.PortraitFrame.LevelBorder:SetWidth(58);
		frame.PortraitFrame.Level:SetText("");
	else
		local follower = C_Garrison.GetFollowerInfo(followerID);
		frame.info = follower;
		local showItemLevel;
		if ( mission.iLevel > 600 and follower.level == GarrisonMissionFrame.followerMaxLevel ) then
			frame.PortraitFrame.LevelBorder:SetAtlas("GarrMission_PortraitRing_iLvlBorder");
			frame.PortraitFrame.LevelBorder:SetWidth(70);
			showItemLevel = true;
		else
			frame.PortraitFrame.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
			frame.PortraitFrame.LevelBorder:SetWidth(58);
			showItemLevel = false;
		end

		GarrisonMissionPlanner_SetFollowerPortrait(frame.PortraitFrame, follower, showItemLevel);
		frame.Class:SetAtlas(follower.classAtlas)
		frame.Class:Show()
		frame.PortraitFrame.Empty:Hide();

		local r,g,b = getFollowerMissionBiasRGB(mission.missionID, follower.followerID);
		frame.PortraitFrame.Level:SetTextColor(r, g, b);
	end

	updateFollowerCounters(frame, getFollowerCounters(mission.missionID, followerID));
end

local function updateMissionMechanics(button)
	local mission = button.info

	local location, xp, environment, environmentDesc, environmentTexture, locPrefix, isExhausting, enemies = C_Garrison.GetMissionInfo(mission.missionID);
	button.MissionEnvIcon.Texture:SetTexture(environmentTexture);
	button.MissionEnvIcon.environment = environment
	button.MissionEnvIcon.environmentDesc = environmentDesc

	local numMechs = 0
	for i = 1, #enemies do
		local enemy = enemies[i]
		for id, mechanic in pairs(enemy.mechanics) do
			numMechs = numMechs + 1;
			if (not button.Mechanics[numMechs]) then
				button.Mechanics[numMechs] = CreateFrame("Button", nil, button, "PlannerMechanicTemplate");
				button.Mechanics[numMechs]:SetPoint("RIGHT", button.Mechanics[numMechs-1], "LEFT", -2, 0);
			end
			local Mechanic = button.Mechanics[numMechs];
			Mechanic.info = mechanic;
			Mechanic.mechanicID = id;
			Mechanic.Texture:SetTexture(mechanic.icon);
			Mechanic:Show();
		end
	end
	for j=(numMechs + 1), #button.Mechanics do
		button.Mechanics[j]:Hide();
		button.Mechanics[j].mechanicID = nil;
		button.Mechanics[j].info = nil;
	end
end

local function setHighlightAlpha(button, alpha)
	button.HighlightT:SetAlpha(alpha)
	button.HighlightB:SetAlpha(alpha)
	button.HighlightTL:SetAlpha(alpha)
	button.HighlightTR:SetAlpha(alpha)
	button.HighlightBL:SetAlpha(alpha)
	button.HighlightBR:SetAlpha(alpha)
	button.Highlight:SetAlpha(alpha)
end

local function updateExpandedDetailsForParty_CheckCounter(button, counterID)
	for e = 1, #button.Enemies do
		local enemy = button.Enemies[e]
		for m = 1, #enemy.Mechanics do
			local mechanic = enemy.Mechanics[m]
			if ( counterID == mechanic.mechanicID and not mechanic.hasCounter ) then
				mechanic.hasCounter = true;
				mechanic.Check:Show();
				return;
			end
		end
	end
end

local function updateExpandedDetailsForParty(button)
	local mission = button.info
	if ( not mission ) then
		return
	end

	local missionID = mission.missionID

	-- MECHANICS
	for e = 1, #button.Enemies do
		local enemy = button.Enemies[e]
		for m = 1, #enemy.Mechanics do
			enemy.Mechanics[m].hasCounter = nil;
			enemy.Mechanics[m].Check:Hide();
		end
	end
	local party = module.parties[missionID] ;
	if ( party ) then
		for i = 1, mission.numFollowers do
			local followerID = party[i]
			if ( followerID and C_Garrison.GetFollowerBiasForMission(missionID, followerID) > -1 ) then
				local abilities = C_Garrison.GetFollowerAbilities(followerID)
				for a = 1, #abilities do
					local ability = abilities[a];
					for counterID, counterInfo in pairs(ability.counters) do
						updateExpandedDetailsForParty_CheckCounter(button, counterID);
					end
				end
			end
		end
	end
end


local function showExpandedDetails(button, mission)
	local numVisibleEnemies = 0;
	local location, xp, environment, environmentDesc, environmentTexture, locPrefix, isExhausting, enemies = C_Garrison.GetMissionInfo(mission.missionID);

	if ( locPrefix ) then
		button.LocBack:SetAtlas("_"..locPrefix.."-Back", true);
		button.LocMid:SetAtlas ("_"..locPrefix.."-Mid", true);
		button.LocFore:SetAtlas("_"..locPrefix.."-Fore", true);

		button.LocBack:Show()
		button.LocMid:Show()
		button.LocFore:Show()
		button.LocBottomBlend:Show()
	end

	-- Realign followers
	local shift = 25
	if ( mission.numFollowers == 1 ) then
		if ( #enemies == 1)  then
			button.Follower1:SetPoint("BOTTOMLEFT", shift + 82, 7);
		else
			button.Follower1:SetPoint("BOTTOMLEFT", shift + 22, 7);
		end
	elseif ( mission.numFollowers == 2 ) then
		button.Follower1:SetPoint("BOTTOMLEFT", shift + 108, 7);
	else
		button.Follower1:SetPoint("BOTTOMLEFT", shift + 22, 7);
	end


	for i = 1, #enemies do
		local Frame = button.Enemies[i];
		if ( not Frame ) then
			break;
		end
		numVisibleEnemies = numVisibleEnemies + 1;
		local enemy = enemies[i];
		Frame.Name:SetText(enemy.name);
		GarrisonEnemyPortait_Set(Frame.PortraitFrame.Portrait, enemy.portraitFileDataID);
		local numMechs = 0;
		for id, mechanic in pairs(enemy.mechanics) do
			numMechs = numMechs + 1;
			if (not Frame.Mechanics[numMechs]) then
				Frame.Mechanics[numMechs] = CreateFrame("Button", nil, Frame, "GarrisonMissionEnemyLargeMechanicTemplate");
				Frame.Mechanics[numMechs]:SetPoint("LEFT", Frame.Mechanics[numMechs-1], "RIGHT", 16, 0);
				Frame.Mechanics[numMechs]:SetScript("OnClick", MissionMinder_FilterFollowersForCounter)
			end
			local Mechanic = Frame.Mechanics[numMechs];
			Mechanic.info = mechanic;
			Mechanic.Icon:SetTexture(mechanic.icon);
			Mechanic.mechanicID = id;
			Mechanic:Show();
		end
		Frame.Mechanics[1]:SetPoint("BOTTOM", (numMechs - 1) * -22, -16);
		for j=(numMechs + 1), #Frame.Mechanics do
			Frame.Mechanics[j]:Hide();
			Frame.Mechanics[j].mechanicID = nil;
			Frame.Mechanics[j].info = nil;
		end
		if ( numMechs > 1 ) then
			Frame.PortraitFrame.Elite:Show();
		else
			Frame.PortraitFrame.Elite:Hide();
		end
		Frame:Show();

		local numEnemies = math.min(#enemies, #button.Enemies)
		if ( numVisibleEnemies == 1 ) then
			button.Enemy1:SetPoint("TOPLEFT", button.Header, "BOTTOMLEFT", 251, -30);
		elseif ( numVisibleEnemies == 2 ) then
			button.Enemy1:SetPoint("TOPLEFT", button.Header, "BOTTOMLEFT", 165, -30);
		else
			button.Enemy1:SetPoint("TOPLEFT", button.Header, "BOTTOMLEFT", 78, -30);
		end
	end

	for i = numVisibleEnemies + 1, #button.Enemies do
		button.Enemies[i]:Hide();
	end
end

local function hideExpandedDetails(button)
	button.LocBack:Hide()
	button.LocMid:Hide()
	button.LocFore:Hide()
	button.LocBottomBlend:Hide()

	for i = 1, #button.Enemies do
		button.Enemies[i]:Hide()
	end
end

local function expandMission(button)
	local listscroll = GarrisonMissionFrame.PlannerTab.listScroll
	button.isExpanded = true
	button:SetHeight(listscroll.collapsedButtonHeight + listscroll.expandedButtonHeight)
	button.IconBG:SetPoint("BOTTOMLEFT", button.Header, 0, 0)
	button.Header:Show()
	button.SelectT:Show()
	button.SelectB:Show()
	button.SelectTL:Show()
	button.SelectTR:Show()
	button.SelectBL:Show()
	button.SelectBR:Show()
	button.MissionTime:Show()
	button.MissionXP:Show()

	button.Chance:SetPoint("CENTER", button.Header, "BOTTOMLEFT", 40, 14);
	button.Rewards[1]:ClearAllPoints()
	button.Rewards[1]:SetPoint("RIGHT", button.Header, "RIGHT", -9, 0)
	for i = 1, #button.Rewards do
		button.Rewards[i]:SetScale(.9)
	end

	button.Summary:Hide()
	button.RareOverlay:Hide()
	button.RareText:Hide()
	button.LocBG:Hide()
	button.MissionEnvIcon:Hide()
	for i = 1, #button.Mechanics do
		button.Mechanics[i]:Hide();
	end
	for i = 1, #button.MinimizedFollowers do
		button.MinimizedFollowers[i]:Hide()
	end

	showExpandedDetails(button, button.info)
end

local function collapseMission(button)
	local listscroll = GarrisonMissionFrame.PlannerTab.listScroll
	button.isExpanded = false
	button:SetHeight(listscroll.collapsedButtonHeight)
	button.IconBG:SetPoint("BOTTOMLEFT", button.ButtonRegion, 0, 1)
	button.Header:Hide()
	button.PartyBuffs:Hide()
	button.SelectT:Hide()
	button.SelectB:Hide()
	button.SelectTL:Hide()
	button.SelectTR:Hide()
	button.SelectBL:Hide()
	button.SelectBR:Hide()
	button.MissionTime:Hide()
	button.MissionXP:Hide()
	for i = 1, #button.Followers do
		button.Followers[i]:Hide()
	end

	button.Rewards[1]:ClearAllPoints()
	button.Rewards[1]:SetPoint("BOTTOMRIGHT", -9, 5)

	button.Summary:Show()
	button.LocBG:Show()
	button.MissionEnvIcon:Show()

	hideExpandedDetails(button)
end

local function updateMissionButton(button)
	local mission = button.info
	if ( mission ) then
		button.Title:SetWidth(0);
		button.Title:SetText(mission.name);

		if ( mission.durationSeconds >= GARRISON_LONG_MISSION_TIME ) then
			local duration = format(GARRISON_LONG_MISSION_TIME_FORMAT, mission.duration);
			button.Summary:SetFormattedText(PARENS_TEMPLATE, duration);
		else
			button.Summary:SetFormattedText(PARENS_TEMPLATE, mission.duration);
		end
		if ( button.Title:GetWidth() + button.Summary:GetWidth() + 8 < 655 - mission.numRewards * 65 ) then
			button.Title:SetPoint("LEFT", 165, 0);
			button.Summary:ClearAllPoints();
			button.Summary:SetPoint("BOTTOMLEFT", button.Title, "BOTTOMRIGHT", 8, 0);
		else
			button.Title:SetPoint("LEFT", 165, 10);
			button.Title:SetWidth(655 - mission.numRewards * 65);
			button.Summary:ClearAllPoints();
			button.Summary:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, -4);
		end
		if ( mission.locPrefix ) then
			button.LocBG:Show();
			button.LocBG:SetAtlas(mission.locPrefix.."-List");
		else
			button.LocBG:Hide();
		end
		if (mission.isRare) then
			button.RareOverlay:Show();
			button.RareText:Show();
			button.IconBG:SetVertexColor(0, 0.012, 0.291, 0.5)
		else
			button.RareOverlay:Hide();
			button.RareText:Hide();
			button.IconBG:SetVertexColor(0, 0, 0, 0.5)
		end

		button.Icon:SetAtlas(mission.typeAtlas)

		local showingItemLevel = false;
		if ( mission.level == GARRISON_FOLLOWER_MAX_LEVEL and mission.iLevel > 0 ) then
			showingItemLevel = true;
		end
		button.Level:SetText(showingItemLevel and mission.iLevel or mission.level);

		button.Chance:SetPoint("CENTER", button.ButtonRegion, "LEFT", 40, (mission.isRare) and -2 or -10);

		for i = 1, mission.numFollowers do
			button.MinimizedFollowers[i]:Show()
			button.Followers[i]:Show();
			if ( not parties[mission.missionID] ) then
				parties[mission.missionID] = {}
			end
			updateFollowerIcon(mission, button.MinimizedFollowers[i], parties[mission.missionID][i]);
			updateFollowerFrame(mission, button.Followers[i], parties[mission.missionID][i]);
		end
		for i = mission.numFollowers + 1, #button.Followers do
			button.MinimizedFollowers[i]:Hide()
			button.Followers[i]:Hide()
		end

		GarrisonMissionPlanner_SetRewards(button, mission.rewards, mission.numRewards);

		--updateFollowersForMission(button)
		updateMissionMechanics(button)
		updateMissionForParty(button)

		if ( module.IsMissionSelected(mission) ) then
			button.SelectedBox:Show();
			setHighlightAlpha(button, 0)
			expandMission(button)
			button:Enable()

			updateExpandedDetailsForParty(button)
		else
			button.SelectedBox:Hide();
			setHighlightAlpha(button, 1)
			collapseMission(button)
			button:Enable()
		end

		button:Show();
	else
		button:Hide();
	end
end

function module.markForUpdate(button)
	button.dirty = true
end

function GarrisonMisisonPlannerButton_OnEnter(self)
	local mission = self.info
	if ( mission and self:IsShown() ) then
		GameTooltip:SetOwner(self, "ANCHOR_NONE");
		GameTooltip:SetPoint("TOPLEFT", self, "TOPRIGHT", 1, 2 )
		GameTooltip:SetText(mission.name)

		local location, baseXP, environment, environmentDesc, environmentTexture, locPrefix, isExhausting, enemies = C_Garrison.GetMissionInfo(mission.missionID)
		local totalTimeString, totalTimeSeconds, isMissionTimeImproved, successChance, partyBuffs, isEnvMechanicCountered, xpBonus, materialMultiplier, goldMultiplier = C_Garrison.GetPartyMissionInfo(mission.missionID)

		-- level
		if ( mission.level == GARRISON_FOLLOWER_MAX_LEVEL and mission.iLevel > 0 ) then
			GameTooltip:AddLine(format(GARRISON_MISSION_LEVEL_ITEMLEVEL_TOOLTIP, mission.level, mission.iLevel), 1, 1, 1)
		else
			GameTooltip:AddLine(format(GARRISON_MISSION_LEVEL_TOOLTIP, mission.level), 1, 1, 1)
		end

		--[===[@debug@
		GameTooltip:AddLine("MissionID = " .. mission.missionID);
		--@end-debug@]===]

		GameTooltip:AddLine(" ")

		-- Offer Time Remaining
		GameTooltip:AddLine(GARRISON_MISSION_AVAILABILITY);
		GameTooltip:AddLine(mission.offerTimeRemaining, 1, 1, 1);

		GameTooltip:Show()
	end
end

local rateBack = 0.1;
local rateMid = 0.3;
local rateFore = 0.8;
function GarrisonMisisonPlannerButton_OnUpdate(self, elapsed)
	if ( self.dirty ) then
		updateMissionButton(self)
		self.dirty = nil
	end

	local changeBack = rateBack/100 * elapsed;
	local changeMid = rateMid/100 * elapsed;
	local changeFore = rateFore/100 * elapsed;

	local backL, _, _, _, backR = self.LocBack:GetTexCoord();
	local midL, _, _, _, midR = self.LocMid:GetTexCoord();
	local foreL, _, _, _, foreR = self.LocFore:GetTexCoord();

	backL = backL + changeBack;
	backR = backR + changeBack;
	midL = midL + changeMid;
	midR = midR + changeMid;
	foreL = foreL + changeFore;
	foreR = foreR + changeFore;

	if (backL >= 1) then
		backL = backL - 1;
		backR = backR - 1;
	end
	if (midL >= 1) then
		midL = midL - 1;
		midR = midR - 1;
	end
	if (foreL >= 1) then
		foreL = foreL - 1;
		foreR = foreR - 1;
	end

	self.LocBack:SetTexCoord(backL, backR, 0, 1);
	self.LocMid:SetTexCoord (midL, midR, 0, 1);
	self.LocFore:SetTexCoord(foreL, foreR, 0, 1);
end

local function addFollowerToMission(event, missionID, index, followerID, source)
	local party = parties[missionID]
	if ( not party ) then
		party = {}
		parties[missionID] = party
	end

	if ( not index ) then -- general, add follower to mission; find first available
		for i = 1, #party + 1 do
			if ( not party[i] ) then
				index = i;
				break;
			end
		end
	end

	party[index] = followerID;
end

local function removeFollowerFromMission(event, missionID, index, source)
	if ( parties[missionID] ) then
		parties[missionID][index] = nil;
	end
end

-- Allow changes on the MissionPage to be reflected in the multiplanner
Lib.RegisterCallback("multiplanner", "FOLLOWER_ADDED_TO_MISSION", addFollowerToMission)
Lib.RegisterCallback("multiplanner", "FOLLOWER_REMOVED_FROM_MISSION", removeFollowerFromMission)
