local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local OPTIONS = Lib.options;
local DEBUG = Lib.debug;

MissionMinder.multiplanner = {};
local module = Lib.multiplanner;

GARRISON_PLANNER = L["Planner"];
GARRISON_PLANNER_START_MISSIONS = L["Start Missions"];
GARRISON_PLANNER_TOO_MANY_ACTIVE_FOLLOWERS = L["Too many active followers"];
GARRISON_PLANNER_NOT_ENOUGH_FOLLOWERS_FOR_CURRENT_MISSIONS = L["Not enough available followers for current missions"];
GARRISON_PLANNER_NO_MISSION_SELECTED = L["No mission has been selected."];

local MULTIPLANNER_TAB = 3

local DEFAULT_MISSION_COST_COLOR = { 1, 1, 1, 1 }
local NOT_ENOUGH_MISSION_COST_COLOR = { 1, 0, 0, 1 }

local missionsReadyToSend = 0;
local readyMissionCost = 0
local missionFollowerCounts = {};

local startMissionTask = nil;
local missionsToSend = {};
local startMissionIndex = 0;

module.state = nil

function module.getState()
	return module.state or {}
end

function module.setState(state, ...)
	if ( module.state ) then
		module.state:destroy()
	end
	module.state = state:New(...)
	module.state:initialize()
	DEBUG("State changed to %s", module.state.name)
end

function module.getSelectedMission()
	local MissionPage = GarrisonMissionFrame.MissionTab.MissionPage
	local PlannerTab = GarrisonMissionFrame.PlannerTab

	if ( PlannerTab:IsShown() and PlannerTab.selectedMission ) then
		return PlannerTab, PlannerTab.selectedMission, PlannerTab.mentorLevel or 0, PlannerTab.mentorILevel or 0
	elseif ( MissionPage:IsVisible() and MissionPage.missionInfo ) then
		return MissionPage, MissionPage.missionInfo, MissionPage.mentorLevel or 0, MissionPage.mentorItemLevel or 0
	end

	return nil, nil, 0, 0
end

function module.setMissionButton(button, missions, index)
	if ( index <= #missions) then
		local mission = missions[index];
		button.id = index;
		button.info = mission;
		button:Show();
	else
		button.id = nil
		button.info = nil
		button:Hide();
	end
end

function module.hideStatusOverlay()
	local self = GarrisonMissionFrame.PlannerTab;
	self.Overlay:Hide();
end

function module.showStatusOverlay(text, ...)
	local self = GarrisonMissionFrame.PlannerTab;
	self.Overlay.Status:SetText(string.format(text, ...));
	self.Overlay:Show();
end

function PlannerMissionList_Update()
	local state = module.getState()
	if ( state.updateMissions ) then
		state:updateMissions()
	end
end


local function updateStartMissionsButton()
	local MissionCost = GarrisonMissionFrame.PlannerTab.MissionCost.Cost
	MissionCost:SetText(readyMissionCost)
	MissionCost:SetTextColor(unpack(DEFAULT_MISSION_COST_COLOR))

	local _, garrisonResources = GetCurrencyInfo(GARRISON_CURRENCY)

	local errorString = nil;
	if ( missionsReadyToSend == 0 ) then
		errorString = GARRISON_PARTY_NOT_FULL_TOOLTIP
	elseif ( garrisonResources < readyMissionCost ) then
		errorString = GARRISON_NOT_ENOUGH_MATERIALS_TOOLTIP
		MissionCost:SetTextColor(unpack(NOT_ENOUGH_MISSION_COST_COLOR))
	end

	local StartMissions = GarrisonMissionFrame.PlannerTab.StartMissions
	if ( errorString ) then
		StartMissions:SetText(GARRISON_PLANNER_START_MISSIONS)
		StartMissions.tooltip = errorString
		StartMissions.Flash:Hide()
		StartMissions.FlashAnim:Stop()
		StartMissions:Disable()
	else
		StartMissions:SetFormattedText("%s (%d)", GARRISON_PLANNER_START_MISSIONS, missionsReadyToSend)
		StartMissions.tooltip = nil
		StartMissions.Flash:Show()
		StartMissions.FlashAnim:Play()
		StartMissions:Enable()
	end
end

local function increaseReadyMissions(mission)
	missionsReadyToSend = missionsReadyToSend + 1
	readyMissionCost = readyMissionCost + mission.cost
	updateStartMissionsButton()
end

local function decreaseReadyMissions(mission)
	missionsReadyToSend = missionsReadyToSend - 1
	readyMissionCost = readyMissionCost - mission.cost
	updateStartMissionsButton()
end

function module.startMission(mission)
	if ( mission ) then
		DEBUG("Sending mission (%d) \"%s\"", mission.missionID, mission.name);
		C_Garrison.StartMission(mission.missionID);
		PlaySound("UI_Garrison_CommandTable_MissionStart");

		decreaseReadyMissions(mission);
	end
end

function GarrisonMissionPlannerTab_OnShow(self)
	GarrisonMissionFrame.FollowerList.showCounters = true;
	GarrisonMissionFrame.FollowerList.canExpand = true;
	GarrisonMissionFrame.FollowerList.showUncollected = false;

	local state = module.getState()
	if ( state.tabShown ) then
		state:tabShown()
	end
end

function GarrisonMissionPlannerTab_OnHide(self)
	GarrisonMissionFrame.FollowerList.showCounters = false;
	GarrisonMissionFrame.FollowerList.canExpand = false;
	GarrisonMissionFrame.FollowerList.showUncollected = false;

	local state = module.getState()
	if ( state.tabShown ) then
		state:tabShown()
	end
end

function GarrisonMissionPlannerStartMissions_OnClick(self, button)
	local state = module.getState()
	if ( state.name == "PLANNER" ) then
		local missionsToSend = {}
		local missions = C_Garrison.GetAvailableMissions();
		for i = 1, #missions do
			local mission = missions[i];
			if ( #mission.followers == mission.numFollowers ) then
				tinsert(missionsToSend, mission);
			end
		end

		if ( #missionsToSend > 0 ) then
			module.setState(module.STARTING_MISSIONS, missionsToSend)
		end
	end
end

local function clearAllParties()
	local missions = C_Garrison.GetAvailableMissions();
	for i = 1, #missions do
		local mission = missions[i];
		module.clearFollowers(mission);
	end
end


function module.updateReadyMissionCount()
	missionsReadyToSend = 0;
	readyMissionCost = 0
	missionFollowerCounts = {};
	updateStartMissionsButton()

	local missions = C_Garrison.GetAvailableMissions();
	for i = 1, #missions do
		local mission = missions[i]
		missionFollowerCounts[mission.missionID] = mission.numFollowers - #mission.followers
		if ( missionFollowerCounts[mission.missionID] == 0 ) then
			increaseReadyMissions(mission)
		end
	end
end


function GarrisonMissionPlanner_ClearFollower(frame, updateValues)
	local followerID = frame.info and frame.info.followerID or nil;
	local mission = frame:GetParent().info;

	frame.info = nil;

	if (followerID) then
		C_Garrison.RemoveFollowerFromMission(mission.missionID, followerID)
		Lib.callbacks:Fire("FOLLOWER_REMOVED_FROM_MISSION", mission.missionID, frame:GetID(), "MultiPlanner")

		-- Update counters (incase removing this follower affects counter opportunities; racial preferences)
		GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(mission.missionID);
		GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(mission.missionID);
		GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);

		if ( missionFollowerCounts[mission.missionID] == 0 ) then
			decreaseReadyMissions(mission)
		end
		missionFollowerCounts[mission.missionID] = missionFollowerCounts[mission.missionID] + 1

		if ( updateValues ) then
			PlaySound("UI_Garrison_CommandTable_UnassignFollower");

			-- update follower list
			GarrisonFollowerList_DirtyList(GarrisonMissionFrame.FollowerList)
			GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
		end
	end

	module.markForUpdate(frame:GetParent());
end


function module.IsMissionSelected(mission)
	local selectedMission = GarrisonMissionFrame.PlannerTab.selectedMission
	return selectedMission and mission and selectedMission.missionID == mission.missionID
end

function GarrisonMissionPlanner_DeselectMission()
	GarrisonMissionFrame.PlannerTab.selectedMission = nil
	Lib.callbacks:Fire("MISSION_SELECTED", nil)
end

function GarrisonMissionPlanner_SelectMission(mission)
	if ( mission ) then
		GarrisonMissionFrame.PlannerTab.selectedMission = mission
		Lib.callbacks:Fire("MISSION_SELECTED", mission.missionID)
	end
end


function GarrisonMissionPlanner_AddFollower(followerID)
	local mission = GarrisonMissionFrame.PlannerTab.selectedMission
	if ( mission and followerID ) then
		if ( not C_Garrison.AddFollowerToMission(mission.missionID, followerID) ) then
			return;
		end
		Lib.callbacks:Fire("FOLLOWER_ADDED_TO_MISSION", mission.missionID, nil, followerID, "MultiPlanner")

		PlaySound("UI_Garrison_CommandTable_AssignFollower");
		-- Update counters (incase adding this follower creates new counter opportunities; racial preferences)
		GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(mission.missionID);
		GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(mission.missionID);
		GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);

		missionFollowerCounts[mission.missionID] = missionFollowerCounts[mission.missionID] - 1
		if ( missionFollowerCounts[mission.missionID] == 0 ) then
			increaseReadyMissions(mission)
		end

		PlannerMissionList_Update()
	end
end

function GarrisonMissionPlanner_SetFollower(frame, follower)
	if (frame.info) then
		GarrisonMissionPlanner_ClearFollower(frame);
	end

	local mission = frame:GetParent().info;
	if ( not C_Garrison.AddFollowerToMission(mission.missionID, follower.followerID) ) then
		return;
	end
	Lib.callbacks:Fire("FOLLOWER_ADDED_TO_MISSION", mission.missionID, frame:GetID(), follower.followerID, "MultiPlanner")

	frame.info = follower;
	PlaySound("UI_Garrison_CommandTable_AssignFollower");
	-- Update counters (incase adding this follower creates new counter opportunities; racial preferences)
	GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(mission.missionID);
	GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(mission.missionID);
	GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);

	missionFollowerCounts[mission.missionID] = missionFollowerCounts[mission.missionID] - 1
	if ( missionFollowerCounts[mission.missionID] == 0 ) then
		increaseReadyMissions(mission)
	end

	-- force button to update immediately
	module.markForUpdate(frame:GetParent());
end

local function updateFollowerListForMission(event, missionID)
	if ( missionID ) then
		GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(missionID)
		GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(missionID)
	else
		GarrisonMissionFrame.followerCounters = nil
		GarrisonMissionFrame.followerTraits = nil
	end

	if ( GarrisonMissionFrame.FollowerList.followers ) then
		GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList)
	end
end


local function selectTab(id)
	if( id == MULTIPLANNER_TAB ) then -- Planner
		PanelTemplates_SetTab(GarrisonMissionFrame, id);

		GarrisonMissionFrame.MissionTab:Hide();
		GarrisonMissionFrame.FollowerTab:Hide();
		GarrisonMissionFrame.PlannerTab:Show()

		if ( GarrisonMissionFrame.FollowerList:IsShown() ) then
			GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
		else
			GarrisonMissionFrame.FollowerList:Show();
		end
		GarrisonMissionFrame.TitleText:SetText(L["Garrison Mission Planner"]);

		if ( UIDropDownMenu_GetCurrentDropDown() == GarrisonFollowerOptionDropDown ) then
			CloseDropDownMenus();
		end
	else -- Mission or Follower
		GarrisonMissionFrame.PlannerTab:Hide()
		module.GarrisonMissionFrame_SelectTab(id)
	end
end

local function selectMissionsTab()
	if (PanelTemplates_GetSelectedTab(GarrisonMissionFrame) == MULTIPLANNER_TAB) then
		if ( GarrisonMissionFrame.MissionTab.MissionPage:IsShown() ) then
			GarrisonMissionPage_Close()
		end
		GarrisonMissionFrame_SelectTab(1); -- Missions
	end
end

local function onMissionFinished(onShow)
	local self = GarrisonMissionFrame;
	if ( self.MissionComplete:IsShown() ) then
		return;
	end

	if ( #self.MissionComplete.completeMissions > 0 and self:IsShown() ) then
		if ( PanelTemplates_GetSelectedTab(self) == MULTIPLANNER_TAB ) then
			GarrisonMissionFrame_SelectTab(1); -- Missions
		end
	end
end


local function disablePlannerTab()
	PanelTemplates_DisableTab(GarrisonMissionFrame, MULTIPLANNER_TAB)
end

local function enablePlannerTab()
	PanelTemplates_EnableTab(GarrisonMissionFrame, MULTIPLANNER_TAB)
end

local function getTopButton(self, offset)
	local selectedMission = GarrisonMissionFrame.PlannerTab.selectedMission or {}
	local missions = C_Garrison.GetAvailableMissions()
	Garrison_SortMissions(missions);

	local totalHeight = 0;
	for i = 1, #missions do
		local height;
		if ( missions[i].missionID == selectedMission.missionID ) then
			height = self.buttonHeight + self.expandedButtonHeight
		else
			height = self.buttonHeight;
		end

		totalHeight = totalHeight + height;
		if ( totalHeight > offset ) then
			return i - 1, height + offset - totalHeight;
		end
	end

	return #missions, 0;
end

function PlannerMissionList_OnLoad(self)
end

local function initialize()
	local self = GarrisonMissionFrame.PlannerTab.listScroll
	self.update = PlannerMissionList_Update
	self.dynamic = function(offset) return getTopButton(self, offset); end
	HybridScrollFrame_CreateButtons(self, "MultiPlannerListButtonTemplate", 5, -4, nil, nil, nil, -4)
	PlannerMissionList_Update()

	local self = GarrisonMissionFrame.PlannerTab
	GarrisonMissionFrame:HookScript("OnHide", clearAllParties);
	GarrisonMissionFrame:HookScript("OnHide", selectMissionsTab);

	for _,button in pairs(self.listScroll.buttons) do
		button:HookScript("OnClick", PlannerMissionList_Update);
	end

	-- Add Planner tab
	module.GarrisonMissionFrame_SelectTab = GarrisonMissionFrame_SelectTab
	GarrisonMissionFrame_SelectTab = selectTab

	-- Disable PlannerTab on CompleteDialog show
	local completeDialog = GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog
	completeDialog:HookScript("OnShow", disablePlannerTab);
	completeDialog:HookScript("OnHide", enablePlannerTab);

	-- Disable PlannerTab on MissionComplete show
	local missionComplete = GarrisonMissionFrame.MissionComplete
	missionComplete:HookScript("OnShow", disablePlannerTab);
	missionComplete:HookScript("OnHide", enablePlannerTab);

	-- Switch to MissionTab on GARRISON_MISSION_FINISHED event to display the CompleteDialog
	hooksecurefunc("GarrisonMissionFrame_CheckCompleteMissions", onMissionFinished)


	PanelTemplates_SetNumTabs(GarrisonMissionFrame, 3);
	PanelTemplates_UpdateTabs(GarrisonMissionFrame);

	module.dragndrop_init()

	-- MissionSelected
	Lib.RegisterCallback("multiplanner", "MISSION_SELECTED", PlannerMissionList_Update);
	Lib.RegisterCallback("multiplanner.followerList", "MISSION_SELECTED", updateFollowerListForMission);

	-- MissionClosed
	Lib.RegisterCallback(Lib, "MissionClosed", PlannerMissionList_Update);


	-- Mark planner tab as beta
	GarrisonMissionFrame.Tab3:SetText("|cff00ff00*BETA*|r " .. GARRISON_PLANNER)


	-- PlannerTab should update followerCounters and Traits for its selected mission
	GarrisonMissionFrame.PlannerTab:HookScript("OnShow", function(self)
		local mission = GarrisonMissionFrame.PlannerTab.selectedMission
		if ( mission ) then
			GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(mission.missionID)
			GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(mission.missionID)
			GarrisonFollowerList_DirtyList(GarrisonMissionFrame.FollowerList)
			GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList)
		else
			GarrisonMissionFrame.followerCounters = nil
			GarrisonMissionFrame.followerTraits = nil
			GarrisonFollowerList_DirtyList(GarrisonMissionFrame.FollowerList)
			GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList)
		end
	end)

	-- Initialize state
	GarrisonMissionFrame.PlannerTab.selectedMission = nil
	module.setState(module.PLANNER)
end

local events = Lib.events
events:RegisterEvent("multiplanner", "ADDON_LOADED", initialize)
events:RegisterEvent("multiplanner", "GARRISON_MISSION_LIST_UPDATE", PlannerMissionList_Update);
